"""Define a client to interact with Pollen.com."""
from typing import Optional
from urllib.parse import ParseResult, urlparse

from aiohttp import ClientSession, ClientTimeout
from aiohttp.client_exceptions import ClientError

from .allergens import Allergens
from .asthma import Asthma
from .disease import Disease
from .errors import InvalidZipError, RequestError

DEFAULT_TIMEOUT: int = 10
DEFAULT_USER_AGENT: str = (
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_4) "
    + "AppleWebKit/537.36 (KHTML, like Gecko) "
    + "Chrome/65.0.3325.181 Safari/537.36"
)


def is_valid_zip_code(zip_code: str) -> bool:
    """Define whether a string ZIP code is valid."""
    return len(zip_code) == 5 and zip_code.isdigit()


class Client:  # pylint: disable=too-few-public-methods
    """Define the client."""

    def __init__(
        self, zip_code: str, *, session: Optional[ClientSession] = None
    ) -> None:
        """Initialize."""
        if not is_valid_zip_code(zip_code):
            raise InvalidZipError(f"Invalid ZIP code: {zip_code}")

        self._session: ClientSession = session
        self.zip_code: str = zip_code

        self.allergens: Allergens = Allergens(self._request)
        self.asthma: Asthma = Asthma(self._request)
        self.disease: Disease = Disease(self._request)

    async def _request(
        self,
        method: str,
        url: str,
        *,
        headers: Optional[dict] = None,
        params: Optional[dict] = None,
        json: Optional[dict] = None,
    ) -> dict:
        """Make a request against AirVisual."""
        pieces: ParseResult = urlparse(url)

        _headers = headers or {}
        _headers.update(
            {
                "Content-Type": "application/json",
                "Referer": f"{pieces.scheme}://{pieces.netloc}",
                "User-Agent": DEFAULT_USER_AGENT,
            }
        )

        use_running_session = self._session and not self._session.closed

        if use_running_session:
            session = self._session
        else:
            session = ClientSession(timeout=ClientTimeout(total=DEFAULT_TIMEOUT))

        try:
            async with session.request(
                method,
                f"{url}/{self.zip_code}",
                headers=_headers,
                params=params,
                json=json,
            ) as resp:
                resp.raise_for_status()
                data: dict = await resp.json(content_type=None)
                return data
        except ClientError as err:
            raise RequestError(f"Error requesting data from {url}: {err}")
        finally:
            if not use_running_session:
                await session.close()



"""Define errors."""


class IQVIAError(Exception):
    """Define a base package error."""


class InvalidZipError(IQVIAError):
    """Define an error when a ZIP returns no valid data."""


class RequestError(IQVIAError):
    """Define a generic request error."""



"""Define an object to work with "Ashtma" endpoints."""
from typing import Awaitable, Callable


class Asthma:  # pylint: disable=too-few-public-methods
    """Define the "Asthma" object."""

    def __init__(self, request: Callable[..., Awaitable[dict]]) -> None:
        """Initialize."""
        self._request: Callable[..., Awaitable[dict]] = request

    async def current(self) -> dict:
        """Get current asthma info."""
        return await self._request(
            "get", "https://www.asthmaforecast.com/api/forecast/current/asthma"
        )

    async def extended(self) -> dict:
        """Get extended asthma info."""
        return await self._request(
            "get", "https://www.asthmaforecast.com/api/forecast/extended/asthma"
        )

    async def historic(self) -> dict:
        """Get historic asthma info."""
        return await self._request(
            "get", "https://www.asthmaforecast.com/api/forecast/historic/asthma"
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyiqvia']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.6.2,<4.0.0']

setup_kwargs = {
    'name': 'pyiqvia',
    'version': '0.3.0',
    'description': 'A clean, async-focused Python3 API for IQVIA data',
    'long_description': '# 🌻 pyiqvia: A clean, async-focused Python3 API for IQVIA™\n\n[![CI](https://github.com/bachya/pyiqvia/workflows/CI/badge.svg)](https://github.com/bachya/pyiqvia/actions)\n[![PyPi](https://img.shields.io/pypi/v/pyiqvia.svg)](https://pypi.python.org/pypi/pyiqvia)\n[![Version](https://img.shields.io/pypi/pyversions/pyiqvia.svg)](https://pypi.python.org/pypi/pyiqvia)\n[![License](https://img.shields.io/pypi/l/pyiqvia.svg)](https://github.com/bachya/pyiqvia/blob/master/LICENSE)\n[![Code Coverage](https://codecov.io/gh/bachya/pyiqvia/branch/dev/graph/badge.svg)](https://codecov.io/gh/bachya/pyiqvia)\n[![Maintainability](https://api.codeclimate.com/v1/badges/3bf37f9cabf73b5d991e/maintainability)](https://codeclimate.com/github/bachya/pyiqvia/maintainability)\n[![Say Thanks](https://img.shields.io/badge/SayThanks-!-1EAEDB.svg)](https://saythanks.io/to/bachya)\n\n`pyiqvia` is an async-focused Python3 library for allergen, asthma, and disease\ndata from the [IQVIA™](https://www.iqvia.com) family of websites (such as \nhttps://pollen.com, https://flustar.com, and more).\n\n- [Python Versions](#python-versions)\n- [Installation](#installation)\n- [Usage](#usage)\n- [Contributing](#contributing)\n\n# Python Versions\n\n`pyiqvia` is currently supported on:\n\n* Python 3.6\n* Python 3.7\n* Python 3.8\n\n# Installation\n\n```python\npip install pyiqvia\n```\n\n# Usage\n\n```python\nimport asyncio\n\nfrom aiohttp import ClientSession\n\nfrom pyiqvia import Client\n\n\nasync def main() -> None:\n    """Run!"""\n    client = Client(80012)\n\n    # ZIP codes starting with 0 need to be provided as strings:\n    client = Client(\'00544\')\n\n    # Get current allergen information:\n    await client.allergens.current()\n\n    # Get more information on the current allergen outlook:\n    await client.allergens.outlook()\n\n    # Get extended forecast allergen information:\n    await client.allergens.extended()\n\n    # Get historic allergen information:\n    await client.allergens.historic()\n\n    # Get current asthma information:\n    await client.asthma.current()\n\n    # Get extended forecast asthma information:\n    await client.asthma.extended()\n\n    # Get historic asthma information:\n    await client.asthma.historic()\n\n    # Get current cold and flu information:\n    await client.disease.current()\n\n    # Get extended forecast cold and flu information:\n    await client.disease.extended()\n\n    # Get historic cold and flu information:\n    await client.disease.historic()\n\n\nasyncio.run(main())\n```\n\nBy default, the library creates a new connection to IQVIA with each coroutine. If you\nare calling a large number of coroutines (or merely want to squeeze out every second of\nruntime savings possible), an\n[`aiohttp`](https://github.com/aio-libs/aiohttp) `ClientSession` can be used for connection\npooling:\n\n```python\nimport asyncio\n\nfrom aiohttp import ClientSession\n\nfrom pyiqvia import Client\n\n\nasync def main() -> None:\n    """Run!"""\n    async with ClientSession() as session:\n        client = Client(80012, session=session)\n\n        # ...\n\n\nasyncio.run(main())\n```\n\n# Contributing\n\n1. [Check for open features/bugs](https://github.com/bachya/pyiqvia/issues)\n  or [initiate a discussion on one](https://github.com/bachya/pyiqvia/issues/new).\n2. [Fork the repository](https://github.com/bachya/pyiqvia/fork).\n3. (_optional, but highly recommended_) Create a virtual environment: `python3 -m venv .venv`\n4. (_optional, but highly recommended_) Enter the virtual environment: `source ./.venv/bin/activate`\n5. Install the dev environment: `script/setup`\n6. Code your new feature or bug fix.\n7. Write tests that cover your new functionality.\n8. Run tests and ensure 100% code coverage: `script/test`\n9. Update `README.md` with any new documentation.\n10. Add yourself to `AUTHORS.md`.\n11. Submit a pull request!\n',
    'author': 'Aaron Bach',
    'author_email': 'bachya1208@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bachya/pyiqvia',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.0,<4.0.0',
}


setup(**setup_kwargs)



"""Define an object to work with "Allergens" endpoints."""
from typing import Awaitable, Callable


class Allergens:
    """Define the "Allergens" object."""

    def __init__(self, request: Callable[..., Awaitable[dict]]) -> None:
        """Initialize."""
        self._request: Callable[..., Awaitable[dict]] = request

    async def current(self) -> dict:
        """Get current allergy conditions."""
        return await self._request(
            "get", "https://www.pollen.com/api/forecast/current/pollen"
        )

    async def extended(self) -> dict:
        """Get extended allergen info."""
        return await self._request(
            "get", "https://www.pollen.com/api/forecast/extended/pollen"
        )

    async def historic(self) -> dict:
        """Get historic allergen info."""
        return await self._request(
            "get", "https://www.pollen.com/api/forecast/historic/pollen"
        )

    async def outlook(self) -> dict:
        """Get allergen outlook."""
        return await self._request("get", "https://www.pollen.com/api/forecast/outlook")
